# encoding: UTF-8
=begin
(c) copyright 2020 TIG

THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, 
INCLUDING WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND 
FITNESS FOR A PARTICULAR PURPOSE.

Name: TIG-DefnSaver

Usage: There are 2 tools...

a. Select one instance of a definition in your model to see the entry, then get...
Context-menu > Save_As... Definition 'Xxxx.skp' [where Xxxx is the name of the selected instance's definition]

b. File menu || Context-menu > Save All Definitions into a Collection...
There is a file-counter displayed in the directory-selector header and the status-bar.

The directory-selector always defaults to the user's Components folder, as set in Preferences...

Version:

1.0 20200914 First release.

=end

###
module TIG
  module DefnSaver
    ### set Constants
    VERSION="1.0"
    ###
    ### Save_As tool
    def self.save_as(defn=nil, name=nil)
      return unless defn
      return unless name
      ### make skp name
      skp = (name+".skp").force_encoding("UTF-8")
      ### get folder
      compos=Sketchup.find_support_file("Components").force_encoding("UTF-8")
      dir = UI.select_directory(
        title: "Select Directory for '#{skp}'...", 
        directory: compos,
        select_multiple: false
      )
      return nil unless dir
      path = File.join(dir, skp)
      defn.save_as(path)
      return true
    end
    ###
    def self.check_ss()
      ss = Sketchup.active_model.selection
      if ss.length==1 && ss[0].is_a?(Sketchup::ComponentInstance)
        inst=ss[0]
        return inst.definition
      else
        return nil
      end
    end
    ###
    def self.fix(name=nil)
      return nil unless name
      name.gsub!(/[\x00\/\\:\*\?\"<>\|#,]/, "_")
      return name
    end
    ###
    unless @loaded
      UI.add_context_menu_handler{|menu|
        defn = self.check_ss()
        if defn
          name=self.fix(defn.name)
          menu.add_item("Save_As... Definition '#{name}.skp'"){self.save_as(defn, name)}
        end
      }
      UI.add_context_menu_handler{|menu|
        menu.add_item("Save All Definitions into a Collection..."){self.save_collection()}
      }
      UI.menu("File").add_submenu("Save All Definitions into a Collection..."){self.save_collection()}
    @loaded = true
    end
    ### Save collection.
    def self.save_collection()
      defns=Sketchup.active_model.definitions.to_a
      defns.dup.each{|defn|
        defns.delete[defn] if defn.image?
        defns.delete[defn] if defn.group?
      }
      ###
      return nil unless defns[0]
      ### choose dir once
      compos=Sketchup.find_support_file("Components").force_encoding("UTF-8")
      dir = UI.select_directory(
        title: "Select Directory for Collection [#{defns.length} files]...", 
        directory: compos,
        select_multiple: false
      )
      return nil unless dir
      ###
      ctr=0
      defns.each{|defn|
        name=self.fix(defn.name)
        ### make skp name
        skp = (name+".skp").force_encoding("UTF-8")
        path = File.join(dir, skp)
        defn.save_as(path)
        ctr+=1
        Sketchup.status_text = "#{ctr} = #{skp}"
      }
      Sketchup.status_text = ""
      Sketchup.send_action("selectSelectionTool:")
      return true
    end
    ###
    
    ###
  end#module
end#module